#!/usr/bin/perl

#
#	USAGE ./script --threshold=<integer> --indir=Alignment_files --outdir=outfiles [--shorten]
#
#	--threshold: a site exclusion threshold (0-100) to exclude unreliably aligned sites from PRANK alignments (requires PRANk xml files)
#	--indir: directory containing input single-gene alignments
#	--outdir: directory for output files
#	a table of protein ID-s which to exclude from the original fasta files (supplied via the %inparalogs data table). 
#	27th march 2014.
#
#	Laszlo G. Nagy
#
#
#

use strict;
use Getopt::Long;
use List::MoreUtils qw(uniq);

my ($threshold, $inputdir, $Outfolder, $shorten);
my (%Matrix, %Probs) = ();
my %inparalogs = ();



###############Parsing command line arguments############
GetOptions( 'indir=s' => \$inputdir, #input directory where the alignment can be found
			'threshold=i' => \$threshold, # exclusion threshold - we found 95 to be good, although higher values would yiled alignments more similar to what you get with frequently used programs such as gblocks
			'outdir=s' => \$Outfolder, 
			'shorten' => \$shorten); 
#			'phyd:s' => \$phylodir, #single copy directoiry
#			'restart' => \$restart); # whether to restart or not

# Check command line parameters
print "$inputdir $threshold $Outfolder\n";
if (not $inputdir or not (-e $inputdir)) { die "ERROR: I don't know where to look for the alignments. Can't guess it, maybe future versions....\n"; }
if (not $threshold) { die "ERROR: you cannot exclude sites without specifying a threshold\n"; }
if (not $Outfolder) { die "ERROR: you havent specified an output folder\n"; }
if ($shorten) {print "Parsimony non-informative gpaaed sites will be excluded\n";}
mkdir $Outfolder;
sleep(1);




opendir(INDIR, $inputdir) || die "can't opendir $inputdir: $!";
while (my $file = readdir(INDIR)) {
	if ($file =~ /\.xml/) {
		%Matrix = %Probs = ();
		my @exclude = ();
		my $infile = $inputdir . '/' . $file;
		print "processing $infile\n";
		my %Mtr = &parse_xml($infile);
		my ($Prb, $length) = &parse_xml_probs($infile);
		foreach (keys %Mtr) {
#			print "faszom $_ $Mtr{$_}\n";
		}
		my %Prb = %$Prb;
		for my $i (0 .. $length) { #for each site in the matrix
			my ($sum, $counter, $av) = 0;
			foreach (keys %Prb) {
				if ($Prb{$_}[$i-1] > 0) { #0 prob is given to gapped sites? by this we make gaps not contribute to the mean of postprobs?
					$sum = $sum + $Prb{$_}[$i-1];
					$counter++;
				}
			}
			if ($counter > 0) {$av = $sum / $counter};
			print "site $i sum is $sum mean is $av div $counter\n";
			if ($av < $threshold) {
				push (@exclude, $i);
			} elsif ($shorten) {
				my @column = ();
				foreach (keys %Mtr)	{
					next if ($Mtr{$_}[$i-1] =~ /-/); #skip gaps, so that the resulting array can be directly used below
					push(@column, $Mtr{$_}[$i-1]);
				}
				if (scalar(@column) < 3) { #this is a pretty lazy way to do it
					print "adding site $i with @column to exclude\n";
					push (@exclude, $i);
				} else {
				print "site $i with @column survived\n";
				}
			} 
		}

		my $NoExcl = scalar(@exclude);
		print "from alignment $infile $NoExcl sites and $inparalogs{$infile} proteins have been excluded \(@exclude\)\n";
		
		my $outfile = $Outfolder . '/' . $file . 'reduced' . $threshold . '.phy';
		open OUT, ">>$outfile" || die "cannot write to this place: $!\n";
		my $newlength = $length - @exclude;
		my $taxlist = my @Tax = keys %Mtr;
		print OUT "$taxlist $newlength\n";
		
		foreach my $row (keys %Mtr) { #key here is the taxname
			my $u = 0;
			open OUT, ">>$outfile" || die "cannot write to this place: $!\n";
			print OUT "$row             ";
			foreach (@{$Mtr{$row}}) {
				$u++; #keeps track of site number
				next if (grep{$_ eq $u}@exclude);
				print "logging $u  acid $_ for species $row \n";
				print OUT "$_";
			}
			print OUT "\n";
		}
	}
}	

###############SUBS##############
sub parse_xml {
	my $infile = shift;
	open INFILE, ">>$infile" || die "cant find: $!\n";
	print INFILE "<leaf\n";
	close INFILE;
	local $/ = '<leaf';
	my $taxname;
	open INFILE, "<$infile" || die "cant find: $!\n";
	while (<INFILE>) {
		chomp;
		if ($_ =~ /sequence/) { #this excludes the first occurrence
			if ($_ =~ /name=\"(.*)\"/) {
				$taxname = $1;
#				print "filename $taxname $infile \n";
				if ($inparalogs{$infile} =~ /$taxname/) {
					print "sequence excluded $taxname\n";
					next;
				} # if the array associated with this filename contains that species then skip sequence
				$taxname =~ s/^\d*_//;
			}
			s/\sid=.*\n.*\n\s*//;
			s/\n\s*\<\/sequence\>\n\<\/leaf\>\n.*//;
			s/\n.*//g;
			my @sequence = split('',$_);
			@{$Matrix{$taxname}} = @sequence;
		}
		if ($_ =~ /probability/) { #this excludes the first occurrence
			if ($_ =~ /name="(.*)"/) {
				$taxname = $1;
			}
			if ($_ =~ /\>\n\s*\<sequence\>\n(.*)\n/) {
				my @sequence = split('',$1);
				@{$Matrix{$taxname}} = @sequence;
			}
		}
	}
	close INFILE;
	return(%Matrix);
}

sub parse_xml_probs {
	my $infile = shift;
	open INFILE, ">>$infile" || die "cant find: $!\n";
	print INFILE "<node\n";
	close INFILE;
	local $/ = '<node';
	my $nodename;
	my $charnum;
	open INFILE, "<$infile" || die "cant find: $!\n";
	while (<INFILE>) {
		chomp;
		if ($_ =~ /probability/) { #this excludes the first occurrence
			if ($_ =~ /id="#(.*)#"/) {
				$nodename = $1;
			}
			if ($_ =~ /\<probability\sid="2"\>\n\s*(.*)\n/) {
				my @probabilities = split(',',$1);
				$charnum = @probabilities;
				for (@probabilities) {
					s/-1|//;
				}
				@{$Probs{$nodename}} = @probabilities;
			}
		}
	}
	close INFILE;
	return(\%Probs, $charnum);
}

